==> ssh-2.2.0.tar.gz: ssh-2.2.0/lib/sshproto/sshtrans.c <==
/*

sshtrans.c

  Authors:
        Tatu Ylonen <ylo@ssh.fi>
        Markku-Juhani Saarinen <mjos@ssh.fi>
        Timo J. Rinne <tri@ssh.fi>
        Sami Lehtinen <sjl@ssh.fi>

  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
  All rights reserved.

  SSH transport layer protocol implementation.  This layer performs
  encryption, integrity, server host authentication, and compression.

*/

/*
 * $Id: sshtrans.c,v 1.21 2000/05/22 10:39:00 sjl Exp $
 * $Log: sshtrans.c,v $
 * $EndLog$
 */

#include "sshincludes.h"
#include "sshtrans.h"
#include "sshcrypt.h"
#include "trcommon.h"
#include "namelist.h"
#include "sshcipherlist.h"

#define SSH_DEBUG_MODULE "Ssh2Trans"

#define DEFAULT_CIPHERS         SSH_STD_CIPHERS
#define DEFAULT_MACS            SSH_STD_MACS
#define DEFAULT_COMPRESSIONS    "none,zlib"
#define DEFAULT_KEXS            "diffie-hellman-group1-sha1"

/* Creates default transport protocol parameter structure.  This structure
   can be modified to choose different parameters. */

SshTransportParams ssh_transport_create_params(void)
{
  SshTransportParams params;
  char *hlp;

  params = ssh_xmalloc(sizeof(*params));
  params->kex_algorithms = ssh_xstrdup(DEFAULT_KEXS);
  hlp = ssh_public_key_get_supported();
  params->host_key_algorithms = ssh_public_key_list_canonicalize(hlp);
  ssh_xfree(hlp);
  params->hash_algorithms = ssh_hash_get_supported();
  params->compressions_c_to_s = ssh_xstrdup(DEFAULT_COMPRESSIONS);
  params->compressions_s_to_c = ssh_xstrdup(DEFAULT_COMPRESSIONS);
  params->compression_level = -1;
  params->ciphers_c_to_s = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
  params->ciphers_s_to_c = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
  params->macs_c_to_s = ssh_name_list_intersection_mac(DEFAULT_MACS);
  params->macs_s_to_c = ssh_name_list_intersection_mac(DEFAULT_MACS);
  return params;
}

/* Frees the protocol parameter structure.  This function should normally not
   be called by applications as the parameters are freed automatically.
   However, if the application gets the parameters just to know what the
   defaults are, it can use this to destroy the parameters. */

void ssh_transport_destroy_params(SshTransportParams params)
{
  ssh_xfree(params->kex_algorithms);
  ssh_xfree(params->host_key_algorithms);
  ssh_xfree(params->hash_algorithms);
  ssh_xfree(params->compressions_c_to_s);
  ssh_xfree(params->compressions_s_to_c);
  ssh_xfree(params->ciphers_c_to_s);
  ssh_xfree(params->ciphers_s_to_c);
  ssh_xfree(params->macs_c_to_s);
  ssh_xfree(params->macs_s_to_c);
  ssh_xfree(params);
}

/* Takes a stream which is supposed to be a connection to the server, and
 * performs client-side processing for the transport layer.  Returns
 * a SshStream object representing the transport layer. */

SshStream ssh_transport_client_wrap(SshStream stream,
                                    const char *application_version,
                                    const char *service,
                                    SshTransportParams params,
                                    const char *server_host_name,
                                    SshKeyCheckCallback key_check,
                                    void *context,
                                    SshVersionCallback version_callback,
                                    void *version_context)
{
  SshTransportCommon tr;

  if (params == NULL)
    params = ssh_transport_create_params();

  tr = ssh_tr_create(stream, FALSE, TRUE, version_callback != NULL,
                     application_version,
                     params);
  tr->version_callback = version_callback;
  tr->version_context = version_context;

  ssh_tr_client_init_kex(tr, service, server_host_name, key_check, context);
  
  return ssh_tr_create_final(tr);
}

/* Takes a stream which is supposed to be a connection to the client,
   and performs server-side processing for the transport layer.  Returns
   a SshStream object representing the transport layer. */

SshStream ssh_transport_server_wrap(SshStream stream,
                                    const char *application_version,
                                    SshTransportParams params,
                                    SshPrivateKey private_host_key,
                                    SshPrivateKey private_server_key,
                                    const unsigned char *public_host_key_blob,
                                    unsigned int public_host_key_blob_len,
                                    SshVersionCallback version_callback,
                                    void *version_context)
{
  SshTransportCommon tr;

  if (params == NULL)
    params = ssh_transport_create_params();

  tr = ssh_tr_create(stream, TRUE, TRUE, version_callback != NULL,
                     application_version,
                     params);
  tr->version_callback = version_callback;
  tr->version_context = version_context;

  ssh_tr_server_init_kex(tr, private_host_key, private_server_key,
                         public_host_key_blob, public_host_key_blob_len);
  
  return ssh_tr_create_final(tr);
}

/* Returns statistics information about the transport layer object. */

void ssh_transport_get_statistics(SshStream stream,
                                  SshTransportStatistics *stats)
{
  /* Verify that this is a transport stream. */
  if (ssh_stream_get_methods(stream) != &ssh_tr_methods)
    {
      memset(stats, 0, sizeof(*stats));
      return;
    }
  
  ssh_transport_get_statistics_tr(ssh_stream_get_context(stream),
                                  stats);
}

void ssh_transport_get_statistics_tr(void *tr_context,
                                     SshTransportStatistics *stats)
{
  SshTransportCommon tr = (SshTransportCommon)tr_context;

  SSH_PRECOND(tr);
  
  /* Return statistics data. */
  stats->compressed_incoming_bytes = tr->compressed_incoming_bytes;
  stats->uncompressed_incoming_bytes = tr->uncompressed_incoming_bytes;
  stats->compressed_outgoing_bytes = tr->compressed_outgoing_bytes;
  stats->uncompressed_outgoing_bytes = tr->uncompressed_outgoing_bytes;
  stats->incoming_packets = tr->incoming_sequence_number;
  stats->outgoing_packets = tr->outgoing_sequence_number;  

  stats->kex_name = tr->kex_name;
  /* Negotiated `default' public key algorithm */
  stats->host_key_name = tr->host_key_name;
  /* List of common host key algorithms */
  stats->host_key_names = tr->host_key_names;

  /* Client to server. */
  stats->cipher_c_to_s = tr->c_to_s.cipher_name;
  stats->mac_c_to_s = tr->c_to_s.mac_name;
  stats->compression_c_to_s = tr->c_to_s.compression_name;

  /* Server to client. */
  stats->cipher_s_to_c = tr->s_to_c.cipher_name;
  stats->mac_s_to_c = tr->s_to_c.mac_name;
  stats->compression_s_to_c = tr->s_to_c.compression_name;
}

/* Return application level compatibility flags. Note that this must
   not be called if tr has become invalid for some reason. The return
   struct should be freed by the caller, when it is no longer
   needed. */
void ssh_transport_get_compatibility_flags(SshStream stream,
                                           SshTransportCompat *compat_flags)
{
  SshTransportCommon tr;
  SshTransportCompat rec;
  
  /* Verify that this is a transport stream. */
  if (ssh_stream_get_methods(stream) != &ssh_tr_methods)
    {
      memset(compat_flags, 0, sizeof(*compat_flags));
      return;
    }

  /* Get the real object. */
  tr = (SshTransportCommon)ssh_stream_get_context(stream);

  rec = ssh_xcalloc(1, sizeof(*rec));
  rec->publickey_draft_incompatibility =
    &(tr->ssh_old_publickey_bug_compat);

  rec->publickey_service_name_draft_incompatibility =
    &(tr->ssh_old_publickey_service_name_bug_compat);
  
  rec->x11_channel_open_draft_incompatibility =
    &(tr->ssh_old_x11_channel_open_bug_compat);

  rec->channel_open_failure_draft_incompatibility =
    &(tr->ssh_old_channel_open_failure_bug_compat);

  rec->publickey_pk_ok_draft_incompatibility =
    &(tr->ssh_old_publickey_pk_ok_msg_bug_compat);
  
  rec->hostbased_service_name_draft_incompatibility =
    &(tr->ssh_old_hostbased_service_name_bug_compat);

  rec->publickey_session_id_encoding_draft_incompatibility =
    &(tr->ssh_old_publickey_session_id_encoding_bug_compat);

  rec->malformed_signatures_draft_incompatibility =
    &(tr->ssh_old_malformed_signatures_bug_compat);

  rec->deprecated_disconnect_codes_draft_incompatibility =
    &(tr->ssh_old_deprecated_disconnect_codes_compat);
  
  *compat_flags = rec;
}

==> ssh-2.4.0.tar.gz: ssh-2.4.0/lib/sshproto/sshtrans.c <==
/*

sshtrans.c

  Authors:
        Tatu Ylonen <ylo@ssh.fi>
        Markku-Juhani Saarinen <mjos@ssh.fi>
        Timo J. Rinne <tri@ssh.fi>
        Sami Lehtinen <sjl@ssh.fi>

  Copyright (C) 1996-2000 SSH Communications Security Oy, Espoo, Finland
  All rights reserved.

  SSH transport layer protocol implementation.  This layer performs
  encryption, integrity, server host authentication, and compression.

*/

/*
 * $Id: sshtrans.c,v 1.24 2000/10/27 13:51:28 gahokas Exp $
 * $Log: sshtrans.c,v $
 * $EndLog$
 */

#include "sshincludes.h"
#include "sshtrans.h"
#include "sshcrypt.h"
#include "trcommon.h"
#include "namelist.h"
#include "sshcipherlist.h"

#define SSH_DEBUG_MODULE "Ssh2Trans"

#define DEFAULT_CIPHERS         SSH_STD_CIPHERS
#define DEFAULT_MACS            SSH_STD_MACS
#define DEFAULT_COMPRESSIONS    "none,zlib"
#define DEFAULT_KEXS            "diffie-hellman-group1-sha1"

/* Creates default transport protocol parameter structure.  This structure
   can be modified to choose different parameters. */

SshTransportParams ssh_transport_create_params(void)
{
  SshTransportParams params;
  char *hlp;

  params = ssh_xmalloc(sizeof(*params));
  params->kex_algorithms = ssh_xstrdup(DEFAULT_KEXS);
  hlp = ssh_public_key_get_supported();
  params->host_key_algorithms = ssh_public_key_list_canonicalize(hlp);
  ssh_xfree(hlp);
  params->hash_algorithms = ssh_hash_get_supported();
  params->compressions_c_to_s = ssh_xstrdup(DEFAULT_COMPRESSIONS);
  params->compressions_s_to_c = ssh_xstrdup(DEFAULT_COMPRESSIONS);
  params->compression_level = -1;
  params->ciphers_c_to_s = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
  params->ciphers_s_to_c = ssh_name_list_intersection_cipher(DEFAULT_CIPHERS);
  params->macs_c_to_s = ssh_name_list_intersection_mac(DEFAULT_MACS);
  params->macs_s_to_c = ssh_name_list_intersection_mac(DEFAULT_MACS);
  return params;
}

/* Frees the protocol parameter structure.  This function should normally not
   be called by applications as the parameters are freed automatically.
   However, if the application gets the parameters just to know what the
   defaults are, it can use this to destroy the parameters. */

void ssh_transport_destroy_params(SshTransportParams params)
{
  ssh_xfree(params->kex_algorithms);
  ssh_xfree(params->host_key_algorithms);
  ssh_xfree(params->hash_algorithms);
  ssh_xfree(params->compressions_c_to_s);
  ssh_xfree(params->compressions_s_to_c);
  ssh_xfree(params->ciphers_c_to_s);
  ssh_xfree(params->ciphers_s_to_c);
  ssh_xfree(params->macs_c_to_s);
  ssh_xfree(params->macs_s_to_c);
  ssh_xfree(params);
}

/* Takes a stream which is supposed to be a connection to the server, and
 * performs client-side processing for the transport layer.  Returns
 * a SshStream object representing the transport layer. */

SshStream ssh_transport_client_wrap(SshStream stream,
                                    const char *application_version,
                                    const char *service,
                                    SshTransportParams params,
                                    const char *server_host_name,
                                    SshKeyCheckCallback key_check,
                                    void *context,
                                    SshVersionCallback version_callback,
                                    void *version_context,
                                    SshRekeyNotifyCallback rekey_notify_cb,
                                    void *rekey_notify_context)
{
  SshTransportCommon tr;

  if (params == NULL)
    params = ssh_transport_create_params();

  tr = ssh_tr_create(stream, FALSE, TRUE, version_callback != NULL,
                     application_version,
                     params);
  tr->version_callback = version_callback;
  tr->version_context = version_context;
  tr->rekey_notify_callback = rekey_notify_cb;
  tr->rekey_notify_context = rekey_notify_context;
  
  ssh_tr_client_init_kex(tr, service, server_host_name, key_check, context);
  
  return ssh_tr_create_final(tr);
}

/* Takes a stream which is supposed to be a connection to the client,
   and performs server-side processing for the transport layer.  Returns
   a SshStream object representing the transport layer. */

SshStream ssh_transport_server_wrap(SshStream stream,
                                    const char *application_version,
                                    SshTransportParams params,
                                    SshPrivateKey private_host_key,
                                    SshPrivateKey private_server_key,
                                    const unsigned char *public_host_key_blob,
                                    unsigned int public_host_key_blob_len,
                                    SshVersionCallback version_callback,
                                    void *version_context,
                                    SshRekeyNotifyCallback rekey_notify_cb,
                                    void *rekey_notify_context)
{
  SshTransportCommon tr;

  if (params == NULL)
    params = ssh_transport_create_params();

  tr = ssh_tr_create(stream, TRUE, TRUE, version_callback != NULL,
                     application_version,
                     params);
  tr->version_callback = version_callback;
  tr->version_context = version_context;
  tr->rekey_notify_callback = rekey_notify_cb;
  tr->rekey_notify_context = rekey_notify_context;

  ssh_tr_server_init_kex(tr, private_host_key, private_server_key,
                         public_host_key_blob, public_host_key_blob_len);
  
  return ssh_tr_create_final(tr);
}

/* Returns statistics information about the transport layer object. */
void ssh_transport_get_statistics(SshStream stream,
                                  SshTransportStatistics stats)
{
  /* Verify that this is a transport stream. */
  if (ssh_stream_get_methods(stream) != &ssh_tr_methods)
    {
      memset(stats, 0, sizeof(*stats));
      return;
    }
  
  ssh_transport_get_statistics_tr(ssh_stream_get_context(stream),
                                  stats);
}

void ssh_transport_get_statistics_tr(void *tr_context,
                                     SshTransportStatistics stats)
{
  SshTransportCommon tr = (SshTransportCommon)tr_context;

  SSH_PRECOND(tr);
  
  /* Return statistics data. */
  stats->compressed_incoming_bytes = tr->compressed_incoming_bytes;
  stats->uncompressed_incoming_bytes = tr->uncompressed_incoming_bytes;
  stats->compressed_outgoing_bytes = tr->compressed_outgoing_bytes;
  stats->uncompressed_outgoing_bytes = tr->uncompressed_outgoing_bytes;
  stats->incoming_packets = tr->incoming_sequence_number;
  stats->outgoing_packets = tr->outgoing_sequence_number;  
  stats->rekeys = tr->successful_rekeys;  

  stats->kex_name = tr->kex_name;
  /* Negotiated `default' public key algorithm */
  stats->host_key_name = tr->host_key_name;
  /* List of common host key algorithms */
  stats->host_key_names = tr->host_key_names;

  /* Client to server. */
  stats->cipher_c_to_s = tr->c_to_s.cipher_name;
  stats->mac_c_to_s = tr->c_to_s.mac_name;
  stats->compression_c_to_s = tr->c_to_s.compression_name;

  /* Server to client. */
  stats->cipher_s_to_c = tr->s_to_c.cipher_name;
  stats->mac_s_to_c = tr->s_to_c.mac_name;
  stats->compression_s_to_c = tr->s_to_c.compression_name;
}

/* Return application level compatibility flags. Note that this must
   not be called if tr has become invalid for some reason. The return
   struct should be freed by the caller, when it is no longer
   needed. */
void ssh_transport_get_compatibility_flags(SshStream stream,
                                           SshTransportCompat *compat_flags)
{
  SshTransportCommon tr;
  SshTransportCompat rec;
  
  /* Verify that this is a transport stream. */
  if (ssh_stream_get_methods(stream) != &ssh_tr_methods)
    {
      memset(compat_flags, 0, sizeof(*compat_flags));
      return;
    }

  /* Get the real object. */
  tr = (SshTransportCommon)ssh_stream_get_context(stream);

  rec = ssh_xcalloc(1, sizeof(*rec));
  rec->publickey_draft_incompatibility =
    &(tr->ssh_old_publickey_bug_compat);

  rec->publickey_service_name_draft_incompatibility =
    &(tr->ssh_old_publickey_service_name_bug_compat);
  
  rec->x11_channel_open_draft_incompatibility =
    &(tr->ssh_old_x11_channel_open_bug_compat);

  rec->channel_open_failure_draft_incompatibility =
    &(tr->ssh_old_channel_open_failure_bug_compat);

  rec->publickey_pk_ok_draft_incompatibility =
    &(tr->ssh_old_publickey_pk_ok_msg_bug_compat);
  
  rec->hostbased_service_name_draft_incompatibility =
    &(tr->ssh_old_hostbased_service_name_bug_compat);

  rec->publickey_session_id_encoding_draft_incompatibility =
    &(tr->ssh_old_publickey_session_id_encoding_bug_compat);

  rec->malformed_signatures_draft_incompatibility =
    &(tr->ssh_old_malformed_signatures_bug_compat);

  rec->deprecated_disconnect_codes_draft_incompatibility =
    &(tr->ssh_old_deprecated_disconnect_codes_compat);

  rec->rekey_incompatibility =
    &(tr->ssh_old_rekey_compat);

  *compat_flags = rec;
}
